@@ -747,9 +747,6 @@ class ReliableMessageDelivery(object):
 
 
     def addOutput(self, output):
-        if not self._connected:
-            self._connected = True
-            self.connectionMade()
         if self._transportlessTimeoutCall is not None:
             self._transportlessTimeoutCall.cancel()
             self._transportlessTimeoutCall = None
@@ -833,6 +830,10 @@ class ReliableMessageDelivery(object):
         exact code path, you should not encounter it.  If you do, something has
         gone *badly* wrong.
         """
+        if not self._connected:
+            self._connected = True
+            self.connectionMade()
+
         ack, incomingMessages = basketCase
 
         outgoingMessages = self.messages
@@ -867,10 +868,6 @@ class ReliableMessageDelivery(object):
                     for (seq, msg) in incomingMessages:
                         if seq > lastSentAck:
                             self.livePage.liveTransportMessageReceived(ctx, msg)
-                        else:
-                            log.msg("Athena transport duplicate message, discarding: %r %r" %
-                                    (self.livePage.clientID,
-                                     seq))
                     d = self._createOutputDeferred()
                 finally:
                     self.unpause()
@@ -1804,7 +1801,7 @@ class _LiveMixin(_HasJSClass, _HasCSSModule):
         """
         if self.fragmentParent is None:
             raise OrphanedFragment(self)
-        for ch in self.liveFragmentChildren:
+        for ch in list(self.liveFragmentChildren):
             ch._athenaDetachServer()
         self.fragmentParent.liveFragmentChildren.remove(self)
         self.fragmentParent = None
