@@ -312,7 +312,8 @@ class IRequest(IComponentized):
     @type args: A mapping of strings (the argument names) to lists of values.
                 i.e., ?foo=bar&foo=baz&quux=spam results in
                 {'foo': ['bar', 'baz'], 'quux': ['spam']}.
-    @ivar received_headers: All received headers.
+    @ivar requestHeaders: Header fields received in the request.
+    @ivar received_headers: DEPRECATED: All received headers.
     """
     method = Attribute("The HTTP method that was used.")
     uri = Attribute("The full URI that was requested (includes arguments).")
@@ -320,11 +321,16 @@ class IRequest(IComponentized):
     prepath = Attribute("Path segments that have already been handled.")
     postpath = Attribute("Path segments still to be handled.")
     args = Attribute("All of the arguments, including URL and POST arguments.")
-    received_headers = Attribute("All received headers.")
+    requestHeaders = Attribute(
+        "A L{http_headers.Headers} instance giving all received HTTP request "
+        "header fields.")
     deferred = Attribute("Fired once request processing is finished.")
     client = Attribute("The client that sent this request.")
     content = Attribute("File-like object containing the request body.")
 
+    # DEPRECATED:
+    received_headers = Attribute("DEPRECATED: All received headers.")
+
     # Methods for received request
     def getHeader(key):
         """Get a header that was sent from the network.
