@@ -25,6 +25,11 @@ Divmod.Runtime.NodeAttributeError.methods(
     });
 
 
+
+Divmod.Runtime.ScriptLoadingError = Divmod.Error.subclass('Divmod.Runtime.ScriptLoadingError');
+
+
+
 /**
  * Wrapper around a lower-level scheduled timed call which provides additional
  * useful operations.
@@ -43,6 +48,102 @@ Divmod.Runtime.DelayedCall.methods(
     });
 
 
+/**
+ * Move the given node so that it is a child of the given document and then try
+ * to find its node with the given id.  Restore the node to its original parent
+ * before returning the result.
+ */
+Divmod.Runtime._getElementByIdWithDocument = function _getElementByIdWithDocument(node, doc, id) {
+    var currentParent = node.parentNode;
+    var nextSibling = node.nextSibling;
+
+    // Insert ourselves into the document temporarily.
+    currentParent.removeChild(node);
+    doc.documentElement.appendChild(node);
+
+    // Try to find it.
+    var foundNode = node.ownerDocument.getElementById(id);
+
+    // And now put us back where we used to be.
+    node.parentNode.removeChild(node);
+    currentParent.insertBefore(node, nextSibling);
+
+    return foundNode;
+};
+
+
+
+/**
+ * Get an element by ID, falling back to using L{_getElementByIdWithDocument}
+ * if it's not found in its owner document.
+ */
+Divmod.Runtime._getElementByIdWithNode = function _getElementByIdWithNode (node, doc, id) {
+    var foundNode = node.ownerDocument.getElementById(id);
+    if (foundNode === null) {
+        // We didn't find it, maybe we need a workaround.
+        // Let's insert ourselves into the document temporarily.
+        foundNode = Divmod.Runtime._getElementByIdWithDocument(node, doc, id);
+    }
+
+    if (foundNode === null) {
+        throw Divmod.Runtime.NodeNotFound('Node with id ' + id + ' not found');
+    }
+
+    return foundNode;
+};
+
+
+
+/**
+ * Register a standard L{Error} JSON serializer.
+ */
+Divmod.Runtime._registerStandardErrorSerializer = function _registerStandardErrorSerializer() {
+    Divmod.Base.registerJSON(
+        'Error',
+        function (obj) {
+            return obj instanceof Error;
+        },
+        function (obj) {
+            return {
+                'name': obj.name,
+                'message': obj.message,
+                'stack': obj.stack || "No stacktrace available\n"
+            };
+        });
+};
+
+
+
+/**
+ * XML parser for platforms which have the builtin "DOMParser" object.
+ */
+Divmod.Runtime._XMLParser = Divmod.Class.subclass("Divmod.Runtime._XMLParser");
+Divmod.Runtime._XMLParser.methods(
+    /**
+     * Return the cached DOM parser for this runtime.  Create one if one does
+     * not yet exist.
+     */
+    function _getParser(self) {
+        if (self._cachedParser === undefined) {
+            self._cachedParser = new DOMParser();
+        }
+        return self._cachedParser;
+    },
+
+    function parseXHTMLString(self, s) {
+        var doc = self._getParser().parseFromString(s, "application/xml");
+        var uri = doc.documentElement.namespaceURI;
+        if (uri != "http://www.w3.org/1999/xhtml") {
+            throw new Error(
+                "Unknown namespace (" + uri +
+                ") used with parseXHTMLString" +
+                "- only XHTML 1.0 is supported.");
+        }
+        return doc;
+    });
+
+
+
 Divmod.Runtime.Platform = Divmod.Class.subclass("Divmod.Runtime.Platform");
 
 Divmod.Runtime.Platform.DOM_DESCEND = 'Divmod.Runtime.Platform.DOM_DESCEND';
@@ -99,13 +200,16 @@ Divmod.Runtime._makeSingleEventHandler = function _makeSingleEventHandler(
     };
 };
 
-
-Divmod.Runtime.Platform.methods(
-    function __init__(self, name) {
+/**
+ * @type _loadEventDelay: Number
+ * @ivar: The number of milliseconds by which to delay invocation of events
+ *     registered with C{addLoadEvent}.
+ */
+Divmod.Runtime.Platform.namedMethods({
+    '__init__': function (self, name) {
         self.name = name;
         self.attrNameToMangled = {};
-        self._scriptCounter = 0;
-        self._scriptDeferreds = {};
+        self._loadEventDelay = 1;
     },
 
     /**
@@ -128,9 +232,8 @@ Divmod.Runtime.Platform.methods(
      *
      * @rtype: C{undefined}
      */
-    function connectSingleDOMEvent(
+    'connectSingleDOMEvent': function (
         self, domEventName, handlerObject, domNode, methodName) {
-
         Divmod.Runtime._eventHandlerObjects[
             handlerObject.__id__] = handlerObject;
 
@@ -156,7 +259,7 @@ Divmod.Runtime.Platform.methods(
      * @throw Error: If there is no node with an attribute which matches the
      * given value.
      */
-    function firstNodeByAttribute(self, root, attrName, attrValue) {
+    'firstNodeByAttribute': function (self, root, attrName, attrValue) {
         /* duplicate this here rather than adding an "onlyOne" arg to
            nodesByAttribute so adding an extra arg accidentally doesn't change
            it's behaviour if called directly
@@ -180,7 +283,7 @@ Divmod.Runtime.Platform.methods(
         return result;
     },
 
-    function nodesByAttribute(self, root, attrName, attrValue) {
+    'nodesByAttribute': function (self, root, attrName, attrValue) {
         var descend = Divmod.Runtime.Platform.DOM_DESCEND;
         var results = [];
         self.traverse(
@@ -194,7 +297,8 @@ Divmod.Runtime.Platform.methods(
         return results;
     },
 
-    function nodeByAttribute(self, root, attrName, attrValue, /* optional */ defaultNode) {
+    'nodeByAttribute': function (self, root, attrName, attrValue,
+                                 /* optional */ defaultNode) {
         var nodes = self.nodesByAttribute(root, attrName, attrValue);
         if (nodes.length > 1) {
             throw new Error("Found too many " + attrName + " = " + attrValue);
@@ -219,7 +323,7 @@ Divmod.Runtime.Platform.methods(
      *
      * @return: object with "x" and "y" slots
      */
-    function getEventCoords(self, event) {
+    'getEventCoords': function (self, event) {
         if(!event) {
             event = window.event;
         }
@@ -244,7 +348,7 @@ Divmod.Runtime.Platform.methods(
      *
      * @rtype: C{Number}
      */
-    function findPosX(self, node) {
+    'findPosX': function (self, node) {
         var curleft = 0;
         if (node.offsetParent) {
             while (node.offsetParent) {
@@ -263,7 +367,7 @@ Divmod.Runtime.Platform.methods(
      *
      * @rtype: C{Number}
      */
-    function findPosY(self, node) {
+    'findPosY': function (self, node) {
         var curtop = 0;
         if (node.offsetParent) {
             while (node.offsetParent) {
@@ -283,8 +387,8 @@ Divmod.Runtime.Platform.methods(
      *
      * @return: object with "w" and "h" attributes
      */
-    function getPageSize(self, /* optional */ win) {
-        var w, h
+    'getPageSize': function (self, /* optional */ win) {
+        var w, h;
         var theWindow = win || window;
 
         /* slightly modified version of code from
@@ -318,7 +422,7 @@ Divmod.Runtime.Platform.methods(
      *
      * @return: object with "w" and "h" attributes
      */
-    function getElementSize(self, e) {
+    'getElementSize': function (self, e) {
         var hidden = e.style.display == "none";
         if(hidden) {
             e.style.display = "";
@@ -333,7 +437,7 @@ Divmod.Runtime.Platform.methods(
     /**
      * Return all immediate children of C{root} that have tag name C{tagName}
      */
-    function getElementsByTagNameShallow(self, root, tagName) {
+    'getElementsByTagNameShallow': function (self, root, tagName) {
         var child, result = [];
         for(var i = 0; i < root.childNodes.length; i++) {
             child = root.childNodes[i];
@@ -351,27 +455,41 @@ Divmod.Runtime.Platform.methods(
      * L{Divmod.Runtime.Platform.setAttribute} when they encounter a
      * namespace-less attribute.
      */
-    function _mangleAttributeName(self, localName) {
+    '_mangleAttributeName': function (self, localName) {
         if(localName in self.attrNameToMangled) {
             return self.attrNameToMangled[localName];
         }
         return localName;
     },
 
-//     function setAttribute(self, node, localName, namespaceURI, namespaceIdentifier, value) {
-//         if (namespaceURI !== undefined && node.setAttributeNS) {
-//             node.setAttributeNS(namespaceURI, localName, value);
-//         } else if (namespaceIdentifier !== undefined) {
-//             node.setAttribute(namespaceIdentifier + ':' + localName, value);
-//         } else {
-//             node.setAttribute(self._mangleAttributeName(localName), value);
-//         }
-//     },
+    /**
+     * Reliably set the value for a node attribute.
+     */
+    'setAttribute': function (self, node, localName, value, namespaceURI,
+                              namespaceIdentifier) {
+        if (namespaceURI === undefined && namespaceIdentifier === undefined) {
+            localName = self._mangleAttributeName(localName);
+        }
+        if (node.hasAttributeNS) {
+            if (node.hasAttributeNS(namespaceURI, localName)) {
+                return node.setAttributeNS(namespaceURI, localName, value);
+            } else if (node.hasAttributeNS(namespaceIdentifier, localName)) {
+                return node.setAttributeNS(namespaceIdentifier, localName, value);
+            }
+        }
+        if (node.setAttribute) {
+            var a = (namespaceIdentifier === undefined) ?
+                localName : namespaceIdentifier + ':' + localName;
+            return node.setAttribute(a, value);
+        }
+        return null;
+    },
 
     /**
      * This is _the_way_ to get the value of an attribute off of node
      */
-    function getAttribute(self, node, localName, namespaceURI, namespaceIdentifier) {
+    'getAttribute': function (self, node, localName, namespaceURI,
+                              namespaceIdentifier) {
         if(namespaceURI == undefined && namespaceIdentifier == undefined) {
             localName = self._mangleAttributeName(localName);
         }
@@ -410,10 +528,12 @@ Divmod.Runtime.Platform.methods(
         return null;
     },
 
-    function makeHTTPRequest(self) {
-        throw new Error("makeHTTPRequest is unimplemented on " + self);
+
+    'makeHTTPRequest': function (self) {
+        return new XMLHttpRequest();
     },
 
+
     /**
      * Asynchronously retrieve an HTTP resource.
      *
@@ -440,7 +560,8 @@ Divmod.Runtime.Platform.methods(
      * code of the response.  For example, 200 if successful.  The 'response'
      * will be a string, the text of the response.
      */
-    function getPage(self, url, /* optional */ args, action, headers, content, synchronous) {
+    'getPage': function (self, url, /* optional */ args, action, headers,
+                         content, synchronous) {
         // Fill out defaults.
         if (args === undefined) {
             args = [];
@@ -499,11 +620,11 @@ Divmod.Runtime.Platform.methods(
         return [req, d];
     },
 
-    function parseXHTMLString(self, s) {
+    'parseXHTMLString': function (self, s) {
         throw new Error("parseXHTMLString not implemented on " + self.name);
     },
 
-    function traverse(self, rootNode, visitor) {
+    'traverse': function (self, rootNode, visitor) {
         if(rootNode == undefined) {
             throw new Error("traverse() passed bad rootNode");
         }
@@ -541,7 +662,7 @@ Divmod.Runtime.Platform.methods(
      * @param node: A DOM node.
      * @param innerHTML The XHTML 1.0 string to append.
      */
-    function appendNodeContent(self, node, innerHTML) {
+    'appendNodeContent': function (self, node, innerHTML) {
         throw new Error("appendNodeContent not implemented on " + self.name);
     },
 
@@ -552,55 +673,40 @@ Divmod.Runtime.Platform.methods(
      * @param node: A DOM node.
      * @param innerHTML The XHTML 1.0 string to append.
      */
-    function setNodeContent(self, node, innerHTML) {
+    'setNodeContent': function (self, node, innerHTML) {
         while (node.childNodes.length) {
             node.removeChild(node.firstChild);
         }
         self.appendNodeContent(node, innerHTML);
     },
 
-    /**
-     * Load the JavaScript module at the given URL.
-     *
-     * The manner in which the JavaScript is evaluated is implementation
-     * dependent.
-     *
-     * @return: A Deferred which fires when the contents of the module have
-     * been evaluated.
-     */
-    function loadScript(self, location) {
-        self._scriptCounter += 1;
-        self._scriptDeferreds[self._scriptCounter] = Divmod.Defer.Deferred();
-
-        var language = '"text/javascript"';
-        var onload = '"Divmod.Runtime.theRuntime._scriptLoaded(' + self._scriptCounter + ')"';
-        var src = '"' + location + '"';
-        var id = '"__athena_runtime_script_loader_' + self._scriptCounter + '__"';
-        var xmlns = '"http://www.w3.org/1999/xhtml"';
-        var script = (
-            '<span ' +
-            'style="display: none" ' +
-            'xmlns=' + xmlns + ' ' +
-            'id=' + id + '>' +
-            '<script ' +
-            'type=' + language + ' ' +
-            'onload=' + onload + ' ' +
-            'src=' + src + '>' +
-            '</script>' +
-            '</span>');
-
-        self.appendNodeContent(document.body, script);
-
-        return self._scriptDeferreds[self._scriptCounter];
+    'loadScript': function (self, location) {
+        // <script> tricks produce spectacularly bizarre behaviour in IE and
+        // Safari doesn't support onerror, so we just use getPage/eval here.
+        var req = Divmod.Runtime.theRuntime.getPage(location);
+        var d = req[1];
+        d.addCallback(
+            function (result) {
+                eval(result['response']);
+            });
+        return d;
     },
 
-    function _scriptLoaded(self, which) {
-        var script = document.getElementById('__athena_runtime_script_loader_' + which + '__');
-        script.parentNode.removeChild(script);
-
-        var loaded = self._scriptDeferreds[which];
-        delete self._scriptDeferreds[which];
-        loaded.callback(null);
+    /**
+     * Load the stylesheet at the given URL by appending a C{link} tag to the
+     * document's head.
+     *
+     * @type stylesheetURL: C{String}
+     *
+     * @rtype: C{undefined}
+     */
+    'loadStylesheet': function (self, location) {
+        var linkNode = document.createElement('link');
+        linkNode.setAttribute('rel', 'stylesheet');
+        linkNode.setAttribute('type', 'text/css');
+        linkNode.setAttribute('href', location);
+        var headNode = document.getElementsByTagName('head')[0];
+        headNode.appendChild(linkNode);
     },
 
     /**
@@ -611,7 +717,7 @@ Divmod.Runtime.Platform.methods(
      * @param deep: A boolean indicating whether children should be imported.
      * @returns: The imported Node.
      */
-    function importNode(self, node, deep) {
+    'importNode': function (self, node, deep) {
         return document.importNode(node, deep);
     },
 
@@ -623,17 +729,67 @@ Divmod.Runtime.Platform.methods(
      * is merely to allow for platfom-specific workarounds for certain edge
      * cases.
      */
-    function getElementByIdWithNode(self, node, id) {
+    'getElementByIdWithNode': function (self, node, id) {
         var foundNode = node.ownerDocument.getElementById(id);
         if (foundNode == null) {
             throw Divmod.Runtime.NodeNotFound('Node with id ' + id + ' not found');
         }
         return foundNode;
-    });
+    },
+
+    /**
+     * Arrange for the given function to be called when the page is fully
+     * loaded.  Due to the behavior of various browsers (particularly with
+     * respect to their "page loading" aka "throbber" user interface element)
+     * this callable is not invoked directly in response to the DOM page load
+     * event, but in a delayed call scheduled from that event.
+     */
+    'addLoadEvent': function (self, callable) {
+        var func = function() { setTimeout(callable, self._loadEventDelay); };
+        Divmod.Base.addToCallStack(window, "onload", func, true);
+    },
+
+    /**
+     * Add a beforeunload event handler.
+     *
+     * @param aWindow: The window object.
+     * @param handler: The handler.
+     */
+    'addBeforeUnloadHandler': function (self, aWindow, handler) {
+        Divmod.Base.addToCallStack(aWindow, 'onbeforeunload', handler);
+    },
 
 
+    /**
+     * Is this a legacy browser?
+     */
+    'isLegacy': function (self) {
+        return false;
+    }
+});
+
+
+/**
+ * Spidermonkey runtime for unit testing.
+ *
+ * @type loadEvents: Array
+ * @ivar loadEvents: A list of load events that have been added.
+ */
 Divmod.Runtime.Spidermonkey = Divmod.Runtime.Platform.subclass(
     'Divmod.Runtime.Spidermonkey');
+Divmod.Runtime.Spidermonkey.methods(
+    function __init__(self, name) {
+        Divmod.Runtime.Spidermonkey.upcall(self, '__init__', name);
+        self.loadEvents = [];
+    },
+
+    /**
+     * Add the given function to loadEvents.
+     */
+    function addLoadEvent(self, callable) {
+        self.loadEvents.push(callable);
+    });
+
 Divmod.Runtime.Spidermonkey.isThisTheOne = function isSpidermonkeyTheOne() {
     try {
         window;
@@ -644,100 +800,8 @@ Divmod.Runtime.Spidermonkey.isThisTheOne = function isSpidermonkeyTheOne() {
 };
 
 
-Divmod.Runtime.XPathSupportingPlatform = Divmod.Runtime.Platform.subclass(
-    'Divmod.Runtime.XPathSupportingPlatform');
-Divmod.Runtime.XPathSupportingPlatform.methods(
-    function _nsResolver(self, prefix) {
-        var ns;
-        switch(prefix) {
-            case 'html':
-                ns = 'http://www.w3.org/1999/xhtml';
-                break;
-            case 'athena':
-                ns = 'http://divmod.org/ns/athena/0.7';
-                break;
-            default:
-                // this should never happen, but browsers still suck...
-                ns = null;
-        }
-        return ns;
-    },
 
-    function _xpathNodeByAttribute(self, attrName, attrValue) {
-        var upper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
-        var lower = upper.toLowerCase();
-        return (
-            ".//*[@*[translate(name(),'" + upper + "', '" + lower + "')='" +
-            attrName + "']='" + attrValue + "'] | .[@*[translate(name(),'" +
-            upper + "', '" + lower + "')='" + attrName + "']='" + attrValue +
-            "']");
-    },
-
-    function firstNodeByAttribute(self, root, attrName, attrValue) {
-        /* duplicate this here rather than adding an "onlyOne" arg to
-           nodesByAttribute so adding an extra arg accidentally doesn't change
-           it's behaviour if called directly
-        */
-        var xpath = self._xpathNodeByAttribute(attrName, attrValue);
-        var node = document.evaluate(
-            xpath,
-            root,
-            null,
-            XPathResult.FIRST_ORDERED_NODE_TYPE,
-            null
-        ).singleNodeValue;
-        if (!node) {
-            throw Divmod.Runtime.NodeAttributeError(root, attrName, attrValue);
-        }
-        return node;
-    },
-
-    function nodeByAttribute(self, root, attrName, attrValue, /* optional */ defaultNode){
-        var xpath = self._xpathNodeByAttribute(attrName, attrValue);
-        var nodes = document.evaluate(
-            xpath,
-            root,
-            null,
-            XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
-            null
-        );
-        if (nodes.snapshotLength > 1) {
-            throw new Error("Found too many " + attrName + " = " + attrValue);
-        }
-        else if (nodes.snapshotLength < 1) {
-            if (defaultNode === undefined) {
-                throw Divmod.Runtime.NodeAttributeError(root, attrName, attrValue);
-            }
-            else {
-                return defaultNode;
-            }
-        }
-        else {
-            var result = nodes.snapshotItem(0);
-            return result;
-        }
-    },
-
-    function nodesByAttribute(self, root, attrName, attrValue) {
-        var results = [];
-        var xpath = self._xpathNodeByAttribute(attrName, attrValue);
-        var nodes = document.evaluate(
-            xpath,
-            root,
-            null,
-            XPathResult.ORDERED_NODE_ITERATOR_TYPE,
-            null
-        );
-        var node = nodes.iterateNext();
-        while(node){
-            results.push(node);
-            node = nodes.iterateNext();
-        }
-        return results;
-    });
-
-
-Divmod.Runtime.Firefox = Divmod.Runtime.XPathSupportingPlatform.subclass(
+Divmod.Runtime.Firefox = Divmod.Runtime.Platform.subclass(
     'Divmod.Runtime.Firefox');
 
 Divmod.Runtime.Firefox.isThisTheOne = function isThisTheOne() {
@@ -755,6 +819,10 @@ Divmod.Runtime.Firefox.isThisTheOne = function isThisTheOne() {
 Divmod.Runtime.Firefox.methods(
     function __init__(self) {
         Divmod.Runtime.Firefox.upcall(self, '__init__', 'Firefox');
+        self._xmlparser = Divmod.Runtime._XMLParser();
+        self._scriptCounter = 0;
+        self._scriptDeferreds = {};
+        Divmod.Runtime._registerStandardErrorSerializer();
     },
 
     function makeHTML(self, element) {
@@ -781,23 +849,25 @@ Divmod.Runtime.Firefox.methods(
         return HTML_ELEMENT;
     },
 
-    /**
-     * Return the cached DOM parser for this runtime.  Create one if one does
-     * not yet exist.
-     */
-    function _getParser(self) {
-        if (self._cachedParser === undefined) {
-            self._cachedParser = new DOMParser();
+
+    function _nsResolver(self, prefix) {
+        var ns;
+        switch (prefix) {
+            case 'html':
+                ns = 'http://www.w3.org/1999/xhtml';
+                break;
+            case 'athena':
+                ns = 'http://divmod.org/ns/athena/0.7';
+                break;
+            default:
+                // this should never happen, but browsers still suck...
+                ns = null;
         }
-        return self._cachedParser;
+        return ns;
     },
 
     function parseXHTMLString(self, s) {
-        var doc = self._getParser().parseFromString(s, "application/xml");
-        if (doc.documentElement.namespaceURI != "http://www.w3.org/1999/xhtml") {
-            throw new Error("Unknown namespace used with parseXHTMLString - only XHTML 1.0 is supported.");
-        }
-        return doc;
+        return self._xmlparser.parseXHTMLString(s);
     },
 
     function appendNodeContent(self, node, innerHTML) {
@@ -830,15 +900,79 @@ Divmod.Runtime.Firefox.methods(
         node.appendChild(document.importNode(doc.documentElement, true));
     },
 
-    function makeHTTPRequest(self) {
-        return new XMLHttpRequest();
+
+    /**
+     * Load the JavaScript module at the given URL.
+     *
+     * The manner in which the JavaScript is evaluated is implementation
+     * dependent.
+     *
+     * @return: A Deferred which fires when the contents of the module have
+     * been evaluated.
+     */
+    function loadScript(self, location) {
+        self._scriptCounter += 1;
+        var scriptID = '__athena_runtime_script_loader_' + self._scriptCounter + '__';
+        self._scriptDeferreds[scriptID] = Divmod.Defer.Deferred();
+
+        var language = '"text/javascript"';
+        var singleQuotedID = "'" + scriptID + "'";
+        var doubleQuotedID = '"' + scriptID + '"';
+        var onload = '"Divmod.Runtime.theRuntime._scriptLoaded(' + singleQuotedID + ')"';
+        var onerror = '"Divmod.Runtime.theRuntime._scriptError(' + singleQuotedID + ', arguments[0])"';
+        var src = '"' + location + '"';
+        var xmlns = '"http://www.w3.org/1999/xhtml"';
+        var script = (
+            '<span ' +
+            'style="display: none" ' +
+            'xmlns=' + xmlns + ' ' +
+            'id=' + doubleQuotedID + '>' +
+            '<script ' +
+            'type=' + language + ' ' +
+            'onload=' + onload + ' ' +
+            'onerror=' + onerror + ' ' +
+            'src=' + src + '>' +
+            '</script>' +
+            '</span>');
+
+        self.appendNodeContent(document.body, script);
+
+        return self._scriptDeferreds[scriptID];
+    },
+
+
+    function _scriptLoaded(self, scriptID) {
+        var script = document.getElementById(scriptID);
+        script.parentNode.removeChild(script);
+
+        var loaded = self._scriptDeferreds[scriptID];
+        delete self._scriptDeferreds[scriptID];
+        loaded.callback(null);
+    },
+
+
+    function _scriptError(self, scriptID, error) {
+        var script = document.getElementById(scriptID);
+        script.parentNode.removeChild(script);
+
+        var loaded = self._scriptDeferreds[scriptID];
+        delete self._scriptDeferreds[scriptID];
+        loaded.errback(Divmod.Runtime.ScriptLoadingError(error));
+    },
+
+
+    function getElementByIdWithNode(self, node, id) {
+        return Divmod.Runtime._getElementByIdWithNode(node, document, id);
     });
 
-Divmod.Runtime.InternetExplorer = Divmod.Runtime.Platform.subclass("Divmod.Runtime.InternetExplorer");
 
-Divmod.Runtime.InternetExplorer.isThisTheOne = function isThisTheOne() {
+
+Divmod.Runtime.WebKit = Divmod.Runtime.Platform.subclass(
+    'Divmod.Runtime.WebKit');
+
+Divmod.Runtime.WebKit.isThisTheOne = function isThisTheOne() {
     try {
-        return navigator.appName == "Microsoft Internet Explorer";
+        return navigator.userAgent.indexOf('WebKit') != -1;
     } catch (err) {
         if (err instanceof ReferenceError) {
             return false;
@@ -848,36 +982,148 @@ Divmod.Runtime.InternetExplorer.isThisTheOne = function isThisTheOne() {
     }
 };
 
-Divmod.Runtime.InternetExplorer.methods(
+Divmod.Runtime.WebKit.methods(
     function __init__(self) {
-        Divmod.Runtime.InternetExplorer.upcall(self, '__init__', 'Internet Explorer');
-        // IE has no equivalent to the stacktrace that FF provides, so this
+        Divmod.Runtime.WebKit.upcall(self, '__init__', 'WebKit');
+        self._xmlparser = Divmod.Runtime._XMLParser();
+        // WebKit has no equivalent to the stacktrace that FF provides, so this
         // JSON adapter will provide a dummy object to make Athena happy when
         // it tries to send exceptions from the client to the server
         Divmod.Base.registerJSON(
             'Error',
             function(obj) {
-                return obj instanceof Error;
+                return arguments[1] instanceof Error;
             },
             function(obj) {
+                var exc = arguments[1];
                 return {
-                    'name': obj.name,
-                    'message': obj.message,
+                    'name': exc.name,
+                    'message': exc.message,
                     'stack': 'No stacktrace available\n'
                 };
             }
         );
-
-        /* IE rewrites attributes with names matching these
-           keys to their corresponding values.
-           e.g. class -> className, etc
+        /*
+         * WebKit fires the onload event long before the page has actually
+         * loaded.  This has two consequences.  First, if the Athena transport
+         * is started when the onload event fires, then WebKit will believe
+         * that the page never actually completes loading, since there will be
+         * an open, long-lived XHR.  This means its spinner will never stop
+         * spinning, which will aggravate users.  Second, if the Athena
+         * transport is started before the page fully loads, application-level
+         * JavaScript may fail, since the page is supposed to have loaded by
+         * the time that event fires.  This delay is a complete and utter hack
+         * to try to work around these two problems in *most* cases.  It will
+         * certainly fail in some cases.  Unless WebKit exposes a real event
+         * which fires when the page is really loaded, though, I don't see any
+         * easy way around this. -exarkun
          */
-        self.attrNameToMangled = {"class": "className",
-                                  "checked": "defaultChecked",
-                                  "usemap": "useMap",
-                                  "for": "htmlFor"};
+        self._loadEventDelay = 200;
     },
 
+    function getElementByIdWithNode(self, node, id) {
+        return Divmod.Runtime._getElementByIdWithNode(node, document, id);
+    },
+
+    function parseXHTMLString(self, s) {
+        return self._xmlparser.parseXHTMLString(s);
+    },
+
+    function appendNodeContent(self, node, innerHTML) {
+        var doc = self.parseXHTMLString(innerHTML);
+        node.appendChild(document.importNode(doc.documentElement, true));
+    });
+
+
+
+Divmod.Runtime._internetExplorerVersion = function _internetExplorerVersion() {
+    try {
+        // http://msdn.microsoft.com/en-us/library/ms537509%28v=vs.85%29.aspx
+        var userAgent = navigator.userAgent;
+        var result = userAgent.match(/MSIE ([0-9]+[\.0-9]*)/);
+        if (result === null && navigator.appName === 'Netscape') {
+            // We might be IE 11.
+            result = userAgent.match(/Trident\/.*rv:([0-9]+[\.0-9])/);
+        }
+        if (result === null) {
+            return null;
+        }
+        return parseFloat(result[1]);
+    } catch (err) {
+        if (err instanceof ReferenceError) {
+            return null;
+        } else {
+            throw err;
+        }
+    }
+};
+
+
+
+Divmod.Runtime.InternetExplorerModern = Divmod.Runtime.Platform.subclass("Divmod.Runtime.InternetExplorerModern");
+
+Divmod.Runtime.InternetExplorerModern.isThisTheOne = function isThisTheOne() {
+    var version = Divmod.Runtime._internetExplorerVersion();
+    return version !== null && version >= 10;
+};
+
+Divmod.Runtime.InternetExplorerModern.methods(
+    function __init__(self) {
+        Divmod.Runtime.InternetExplorerModern.upcall(
+            self, '__init__', 'Internet Explorer 10+');
+        self._xmlparser = Divmod.Runtime._XMLParser();
+    },
+
+
+    function appendNodeContent(self, node, innerHTML) {
+        var head = document.getElementsByTagName('head').item(0);
+        var doc = self.parseXHTMLString(innerHTML);
+        var scripts = doc.getElementsByTagName('script');
+
+        while (scripts.length > 0) {
+            var oldScript = scripts[0].parentNode.removeChild(scripts[0]);
+            var src = oldScript.getAttribute('src');
+            var text = oldScript.text;
+            var script = document.createElement('script');
+            script.type = 'text/javascript';
+            if(src != '' && src != null){
+                script.src = src;
+            }
+            else if(text != '' && text != null){
+                script.text = text;
+            }
+            head.appendChild(script);
+        }
+        node.appendChild(document.importNode(doc.documentElement, true));
+    },
+
+
+    function parseXHTMLString(self, s) {
+        return self._xmlparser.parseXHTMLString(s);
+    },
+
+
+    function getElementByIdWithNode(self, node, id) {
+        return Divmod.Runtime._getElementByIdWithNode(node, document, id);
+    });
+
+
+
+Divmod.Runtime.InternetExplorerSemiModern = Divmod.Runtime.Platform.subclass("Divmod.Runtime.InternetExplorerSemiModern");
+
+Divmod.Runtime.InternetExplorerSemiModern.isThisTheOne = function isThisTheOne() {
+    var version = Divmod.Runtime._internetExplorerVersion();
+    return version !== null && version >= 8 && version < 10;
+};
+
+Divmod.Runtime.InternetExplorerSemiModern.methods(
+    function __init__(self) {
+        Divmod.Runtime.InternetExplorerSemiModern.upcall(
+            self, '__init__', 'Internet Explorer 8/9');
+        Divmod.Runtime._registerStandardErrorSerializer();
+    },
+
+
     function parseXHTMLString(self, s) {
         var xmldoc = new ActiveXObject("MSXML.DOMDocument");
         xmldoc.async = false;
@@ -888,6 +1134,7 @@ Divmod.Runtime.InternetExplorer.methods(
         return xmldoc;
     },
 
+
     function appendNodeContent(self, node, innerHTML) {
         var head = document.getElementsByTagName('head').item(0);
         var doc = self.parseXHTMLString(innerHTML);
@@ -911,37 +1158,6 @@ Divmod.Runtime.InternetExplorer.methods(
         node.innerHTML += doc.xml;
     },
 
-    function makeHTTPRequest(self) {
-        if (!self._xmlhttpname) {
-            var names = ["Msxml2.XMLHTTP", "Microsoft.XMLHTTP", "Msxml2.XMLHTTP.4.0"];
-            while (names.length) {
-                self._xmlhttpname = names.shift();
-                try {
-                    return self.makeHTTPRequest();
-                } catch (e) {
-                    // pass
-                }
-            }
-            self._xmlhttpname = null;
-            throw new Error("No support XML HTTP Request thingy on this platform");
-        } else {
-            return new ActiveXObject(self._xmlhttpname);
-        }
-    },
-
-    function loadScript(self, location) {
-        // The <script> tricks the base implementation uses produce
-        // spectacularly bizarre behaviour in IE, so we just use getPage/eval
-        // here.
-
-        var req = Divmod.Runtime.theRuntime.getPage(location);
-        var d = req[1];
-        d.addCallback(
-            function (result) {
-                eval(result['response']);
-            });
-        return d;
-    },
 
     /**
      * Simulate importNode by rebuilding the node contents in a node created in
@@ -960,6 +1176,7 @@ Divmod.Runtime.InternetExplorer.methods(
         return tmpNode.firstChild.cloneNode(deep);
     },
 
+
     function getElementByIdWithNode(self, node, id) {
         var foundNode = node.ownerDocument.getElementById(id);
         if (foundNode == null) {
@@ -974,19 +1191,9 @@ Divmod.Runtime.InternetExplorer.methods(
             var DOCUMENT_FRAGMENT_NODE = 11;
             if (root.nodeType == DOCUMENT_FRAGMENT_NODE) {
                 // We're in a DocumentFragment, and thus getElementById won't
-                // find any of our children. Let's insert ourselves into the
-                // document temporarily.
-                var currentParent = node.parentNode;
-                var nextSibling = node.nextSibling;
-                currentParent.removeChild(node);
-                document.documentElement.appendChild(node);
-
-                // Try again
-                foundNode = node.ownerDocument.getElementById(id);
-
-                // And now put us back where we used to be.
-                node.parentNode.removeChild(node);
-                currentParent.insertBefore(node, nextSibling);
+                // find any of our children.
+                foundNode = Divmod.Runtime._getElementByIdWithDocument(
+                    node, document, id);
             }
         }
 
@@ -995,10 +1202,66 @@ Divmod.Runtime.InternetExplorer.methods(
         }
 
         return foundNode;
+    },
+
+
+    /**
+     * Add the handler to the <body> element instead.
+     */
+    function addBeforeUnloadHandler(self, aWindow, handler) {
+        Divmod.Base.addToCallStack(aWindow.document.body, 'onbeforeunload', handler);
     });
 
 
-Divmod.Runtime.Opera = Divmod.Runtime.XPathSupportingPlatform.subclass("Divmod.Runtime.Opera");
+
+Divmod.Runtime.InternetExplorerLegacy = Divmod.Runtime.InternetExplorerSemiModern.subclass("Divmod.Runtime.InternetExplorerLegacy");
+
+Divmod.Runtime.InternetExplorerLegacy.isThisTheOne = function isThisTheOne() {
+    var version = Divmod.Runtime._internetExplorerVersion();
+    return version !== null && version < 8;
+};
+
+Divmod.Runtime.InternetExplorerLegacy.methods(
+    function __init__(self) {
+        Divmod.Runtime.InternetExplorerLegacy.upcall(
+            self, '__init__', 'Internet Explorer 6/7');
+        /* IE rewrites attributes with names matching these
+           keys to their corresponding values.
+           e.g. class -> className, etc
+         */
+        self.attrNameToMangled = {"class": "className",
+                                  "checked": "defaultChecked",
+                                  "usemap": "useMap",
+                                  "for": "htmlFor"};
+    },
+
+
+    function isLegacy(self) {
+        return true;
+    },
+
+
+    function makeHTTPRequest(self) {
+        if (!self._xmlhttpname) {
+            var names = ["Msxml2.XMLHTTP", "Microsoft.XMLHTTP", "Msxml2.XMLHTTP.4.0"];
+            while (names.length) {
+                self._xmlhttpname = names.shift();
+                try {
+                    return self.makeHTTPRequest();
+                } catch (e) {
+                    // pass
+                }
+            }
+            self._xmlhttpname = null;
+            throw new Error("No support XML HTTP Request thingy on this platform");
+        } else {
+            return new ActiveXObject(self._xmlhttpname);
+        }
+    });
+
+
+
+Divmod.Runtime.Opera = Divmod.Runtime.Platform.subclass("Divmod.Runtime.Opera");
 
 Divmod.Runtime.Opera.isThisTheOne = function isThisTheOne() {
     try {
@@ -1053,18 +1316,17 @@ Divmod.Runtime.Opera.methods(
     function appendNodeContent(self, node, innerHTML) {
         var doc = self.parseXHTMLString(innerHTML);
         node.appendChild(document.importNode(doc.documentElement, true));
-    },
-
-    function makeHTTPRequest(self) {
-        return new XMLHttpRequest();
     });
 
 
 
 Divmod.Runtime.Platform.determinePlatform = function determinePlatform() {
     var platforms = [
+        Divmod.Runtime.WebKit,
+        Divmod.Runtime.InternetExplorerModern,
         Divmod.Runtime.Firefox,
-        Divmod.Runtime.InternetExplorer,
+        Divmod.Runtime.InternetExplorerSemiModern,
+        Divmod.Runtime.InternetExplorerLegacy,
         Divmod.Runtime.Opera,
         Divmod.Runtime.Spidermonkey];
     for (var cls = 0; cls < platforms.length; ++cls) {
@@ -1075,5 +1337,19 @@ Divmod.Runtime.Platform.determinePlatform = function determinePlatform() {
     throw new Error("Unsupported platform");
 };
 
-Divmod.Runtime.theRuntimeType = Divmod.Runtime.Platform.determinePlatform();
-Divmod.Runtime.theRuntime = new Divmod.Runtime.theRuntimeType;
+/**
+ * Initialise the Divmod runtime.
+ */
+Divmod.Runtime.initRuntime = function initRuntime(runtimeType) {
+    // If the runtime type has already been determined, then we do not attempt
+    // detection again. The platform type shouldn't be changing at runtime, and
+    // the detection heuristics can be broken by code that executes later.
+
+    if (Divmod.Runtime.theRuntimeType === undefined) {
+        Divmod.Runtime.theRuntimeType = Divmod.Runtime.Platform.determinePlatform();
+    }
+
+    Divmod.Runtime.theRuntime = new Divmod.Runtime.theRuntimeType;
+}
+
+Divmod.Runtime.initRuntime();
