@@ -35,6 +35,75 @@ Nevow.Test.TestWidget.WidgetTests.methods(
         self._unmockRDM();
     },
 
+    /**
+     * L{Nevow.Athena.Widget.addChildWidgetFromWidgetInfo}'s deferred should
+     * errback if one of the import deferreds does.
+     */
+    function test_addChildWidgetFromWidgetInfoError(self) {
+        var requiredModules = [
+            ['test_addChildWidgetFromWidgetInfoError1',
+             '/test_addChildWidgetFromWidgetInfoError1']];
+        var widgetInfo = {
+            requiredModules: requiredModules,
+            requiredCSSModules: [],
+            id: 'test_addChildWidgetFromWidgetInfoError',
+            'class': 'test_addChildWidgetFromWidgetInfoError',
+            children: [],
+            initArguments: [],
+            markup: ''};
+        var loadScriptError = new Error(
+            'test_addChildWidgetFromWidgetInfoloadScriptError');
+        var origLoadScript = Divmod.Runtime.theRuntime.loadScript;
+        var loadScript = function loadScript(location) {
+            self.assertIdentical(location, requiredModules[0][1]);
+            return Divmod.Defer.fail(loadScriptError);
+        }
+        Divmod.Runtime.theRuntime.loadScript = loadScript;
+        try {
+            var result = self.widget.addChildWidgetFromWidgetInfo(widgetInfo);
+        } finally {
+            Divmod.Runtime.theRuntime.loadScript = origLoadScript;
+        }
+        var theFailure;
+        result.addErrback(
+            function(err) {
+                theFailure = err;
+            });
+        var firstError = theFailure.check(Divmod.Defer.FirstError);
+        self.assertIdentical(firstError.err.error, loadScriptError);
+    },
+
+    /**
+     * L{Nevow.Athena.Widget.addChildWidgetFromWidgetInfo} should call
+     * L{Divmod.Runtime.Platform.loadStylesheet} with each required
+     * stylesheet.
+     */
+    function test_addChildWidgetStylesheets(self) {
+        var requiredCSSModules = [
+            '/test_addChildWidgetStylesheets1',
+            '/test_addChildWidgetStylesheets2'];
+        var widgetInfo = {
+            requiredModules: [],
+            requiredCSSModules: requiredCSSModules,
+            id: 'test_addChildWidgetStylesheets',
+            'class': 'test_addChildWidgetStylesheets',
+            children: [],
+            initArguments: [],
+            markup: ''};
+        var loadedStylesheets = [];
+        var origLoadStylesheet = Divmod.Runtime.theRuntime.loadStylesheet;
+        var loadStylesheet = function loadStylesheet(location) {
+            loadedStylesheets.push(location);
+        }
+        Divmod.Runtime.theRuntime.loadStylesheet = loadStylesheet;
+        try {
+            var result = self.widget.addChildWidgetFromWidgetInfo(widgetInfo);
+        } finally {
+            Divmod.Runtime.theRuntime.loadStylesheet = loadStylesheet;
+        }
+        self.assertArraysEqual(loadedStylesheets, requiredCSSModules);
+    },
+
     /**
      * Verify that translateNodeId returns a correctly translated id.
      */
@@ -311,18 +380,17 @@ Nevow.Test.TestWidget.SetupTests.methods(
     },
 
     /**
-     * Verify that the window's onbeforeunload event will set the pageUnloaded
-     * attribute and invoke the delivery channel's fast-path instant teardown.
+     * Verify that the page's onbeforeunload event handler will set the
+     * pageUnloaded attribute and invoke the delivery channel's fast-path
+     * instant teardown.
      */
     function test_onbeforeunload(self) {
-        var notAWindow = {};
-        self.page.bindEvents(notAWindow);
         self.assertIdentical(self.page.pageUnloaded, false);
         var disconnectDialogShown = false;
         self.page.showDisconnectDialog = function () {
             disconnectDialogShown = true;
         };
-        notAWindow.onbeforeunload();
+        self.page.onbeforeunload();
         self.assertIdentical(self.page.pageUnloaded, true);
         self.assertIdentical(self.closeMessageSent, true);
         // This should not show the disconnect dialog; onbeforeunload is a
@@ -412,7 +480,8 @@ Nevow.Test.TestWidget.SetupTests.methods(
         self.page.onkeypress = function () {
             self.called = true;
         };
-        self.page.bindEvents(self.window);
+        Divmod.Base.addToCallStack(
+            self.window, 'onkeypress', self.page.makeHandler('onkeypress'));
         self.window.onkeypress(fakeEvt);
         self.assert(self.called, "Key not bound.")
     });
