@@ -8,8 +8,10 @@ Tests for L{nevow.appserver}.
 from zope.interface import implements
 
 from cStringIO import StringIO
+from shlex import split
 
 from twisted.trial.unittest import TestCase
+from twisted.internet.defer import Deferred
 
 from nevow import inevow
 from nevow import appserver
@@ -95,6 +97,7 @@ class TestLookup(testutil.TestCase):
             asserterr)
 
 
+
 class TestSiteAndRequest(testutil.TestCase):
     def renderResource(self, resource, path):
         s = appserver.NevowSite(resource)
@@ -130,6 +133,44 @@ class TestSiteAndRequest(testutil.TestCase):
         return self.renderResource(Res1(), 'bar').addCallback(
             lambda result: self.assertEquals(result, 'world'))
 
+    def test_connectionLost(self):
+        """
+        L{Request.finish} is not called when the connection is lost before
+        rendering has finished.
+        """
+        rendering = Deferred()
+        class Res(Render):
+            def renderHTTP(self, ctx):
+                return rendering
+        site = appserver.NevowSite(Res())
+        request = appserver.NevowRequest(testutil.FakeChannel(site), True)
+        request.connectionLost(Exception("Just Testing"))
+        rendering.callback(b"finished")
+
+        self.assertFalse(
+            request.finished, "Request was incorrectly marked as finished.")
+
+
+    def test_renderPOST(self):
+        """
+        A POST request with form data has the form data parsed into
+        C{request.fields}.
+        """
+        class Res(Render):
+            def renderHTTP(self, ctx):
+                return b''
+
+        s = appserver.NevowSite(Res())
+        r = appserver.NevowRequest(
+            testutil.FakeChannel(s), True)
+        r.method = b'POST'
+        r.path = b'/'
+        r.content = StringIO(b'foo=bar')
+        self.successResultOf(r.process())
+        self.assertEquals(r.fields[b'foo'].value, b'bar')
+
+
+
 from twisted.internet import protocol, address
 
 class FakeTransport(protocol.FileWrapper):
@@ -187,11 +228,10 @@ class Logging(testutil.TestCase):
         proto = self.renderResource('/foo')
         logLines = proto.site.logFile.getvalue().splitlines()
         self.assertEquals(len(logLines), 1)
-        # print proto.transport.data.getvalue()
-        self.assertEquals(
-            logLines,
-            ['"fakeaddress2" - - faketime "GET /foo HTTP/1.0" 200 6 '
-             '"fakerefer" "fakeagent"'])
+        self.assertEqual(
+            split(logLines[0]),
+            ['fakeaddress2', '-', '-', 'faketime', 'GET /foo HTTP/1.0', '200', '6',
+             'fakerefer', 'fakeagent'])
 
 
     def test_newStyle(self):
