@@ -4,8 +4,9 @@
 
 from twisted.internet import defer
 
-from nevow import context
+from nevow import context, inevow
 from nevow import testutil
+from nevow.flat import twist
 from nevow.util import Deferred
 
 from nevow import rend, loaders, tags
@@ -66,6 +67,50 @@ class LaterRenderTest(RenderHelper):
         self.d2.callback(".")
         self.assertEquals(req.v, '<html>Hello world.</html>')
 
+    def test_renderNestedDeferredCallables(self):
+        """
+        Test flattening of a renderer which returns a Deferred which fires with
+        a renderer which returns a Deferred.
+        """
+        def render_inner(ctx, data):
+            return defer.succeed('')
+
+        def render_outer(ctx, data):
+            return defer.succeed(render_inner)
+
+        ctx = context.WovenContext()
+        ctx.remember(None, inevow.IData)
+
+        out = []
+        d = twist.deferflatten(render_outer, ctx, out.append)
+        def flattened(ign):
+            self.assertEquals(out, [''])
+        d.addCallback(flattened)
+        return d
+
+
+    def test_renderNestedDeferredErrorHandling(self):
+        """
+        Test that flattening a renderer which returns a Deferred which fires
+        with a renderer which raises an exception causes the outermost Deferred
+        to errback.
+        """
+        class NestedException(Exception):
+            pass
+
+        def render_inner(ctx, data):
+            raise NestedException()
+
+        def render_outer(ctx, data):
+            return defer.succeed(render_inner)
+
+        ctx = context.WovenContext()
+        ctx.remember(None, inevow.IData)
+
+        out = []
+        d = twist.deferflatten(render_outer, ctx, out.append)
+        return self.assertFailure(d, NestedException)
+
 
 class LaterDataTest(RenderHelper):
     def data_later(self, context, data):
@@ -93,7 +138,11 @@ class LaterDataTest(RenderHelper):
 
 
 class SuperLaterDataTest(RenderHelper):
-    def test_realDeferredSupport(self):
+    def test_reusedDeferredSupport(self):
+        """
+        Two occurrences of a particular slot are each replaced with the
+        result of the Deferred which is used to fill that slot.
+        """
         doc = tags.html[
             tags.slot('foo'), tags.slot('foo')]
         doc.fillSlots('foo', defer.succeed(tags.span['Foo!!!']))
@@ -108,14 +157,14 @@ class SuperLaterDataTest(RenderHelper):
         render function is called only once.
         """
         calls = []
-        def renderer(ctx, data):
+        def recorder(ctx, data):
             calls.append(None)
             return str(len(calls))
         doc = tags.html[tags.directive('renderer')]
         class RendererPage(rend.Page):
             docFactory = loaders.stan(doc)
             def render_renderer(self, ctx, data):
-                return defer.succeed(renderer)
+                return defer.succeed(recorder)
         self.r = RendererPage()
         req = self.renderIt()
         self.assertEquals(req.v, '<html>1</html>')
